PRO SUV
compile_opt hidden
on_error,1

;...initialization
!quiet=1 & quiet=check_math(0,1) & restore,'track.idl'
jxx=201 & jx=jxx-1 & s=1e-1*indgen(jxx)
vgx=sqrt(-1) & uvstr={min:-1,alf:8e-2,ur:s,sg:s,vt:s,vg:s,gg:s,dd:s,$
d2ur:s,d2sg:s,d2vt:s,d2vg:s,d2gg:s,d2dd:s}

;...calculate radial and tangential wind profiles for each fix
for k=1,entries do begin
delp=pdf(k-1) & rmw=rmx(k-1) & fbz=fbr(k-1) & wr=-1 & wl=-1 
bhl=hdb(k-1) & if bhl eq 0. then begin  bhl=1.38+1.84e-3*delp-3.09e-3*rmw
hdb(k-1)=bhl &endif 

;...gradient wind profile (modified holland)
VGHGEN,s,delp,bhl,fbz,vg,d2vg,alp,vgz 
vort=deriv(s,vg)+vg/s & vort(0)=vort(1) 
dd=vort+fbz/vgz & gg=2*vg/s+fbz/vgz & gg(0)=gg(1) 
d2dd=spl_init(s,dd,yp0=0.) & d2gg=spl_init(s,gg,yp0=0.)
alfi=-6.03871e-2/delp+1.64839e-3*rmw+6.41787e-3*fbz+1.77758/vgz
; This part defines the friction term 
if (lsflg(k-1) eq 3) then begin
	sigma=0.9 &$
; on land h is 1000m
	h=1000.0 &$
; on land cd=0.01 (based on Zd=0.2)
	cd=0.01 &$
endif else begin
	sigma=0.3 &$
; on water h is 450m
	h=450.0  &$
; on water we are using the following formula (Large and Pond- 1981)
        cd=(0.49+0.065*(0.8*vgz))*0.001 &$
        if ((0.8*vgz) ge 34 ) then cd=0.002 &$
endelse
; alff will go in the main equation, and alfi is used in 
; in the ODE.
alff=sigma*1000.0*((rmw*cd)/h)

;...momentum equations furnish tangential and radial profiles
while wl(0) eq -1 or wr(0) eq -1 do begin 
USG,s,sga,ua,va,wa,vg,d2vg,gg,d2gg,dd,d2dd,alfi,iw 
if iw lt 10 then begin wl=wa & vl=va & ul=ua & alfi=alfi-.01 &endif 
if iw gt 10 then begin wr=wa & vr=va & ur=ua & alfi=alfi+.01 &endif 
if (iw ge 7 and iw le 10)  then begin 
v=va & u=ua & wl=1 & wr=1 & endif & endwhile

sg=v-vg & w=sqrt(u^2+v^2) 

;...spline fitting for later use
if min(u) ge 0. then begin u=-abs(u) & sg(*)=0. & v=vg & w=sqrt(u^2+v^2) & endif
d2u=spl_init(s,u,yp0=0) & d2sg=spl_init(s,sg,yp0=0) & d2v=spl_init(s,v)
vgx=[vgx,vgz]

;...display profiles for each fix
;!x.margin=[3,1] & !y.margin=[2,2] & !x.style=1 &!y.style=1 & !x.range=[0,15] & !y.range=[0,1.4]
;plot,s,w,title=string(k) & oplot,s,-u,psym=3 & oplot,s,v,psym=3  
;oplot,s,vg,thick=2,linestyle=3 & wait,5

;...collect profiles into structure
uvstr=[uvstr,{min:index(k-1),alf:alff,ur:u,sg:sg,vt:v,vg:vg,gg:gg,dd:dd,$
     d2ur:d2u,d2sg:d2sg,d2vt:d2v,d2vg:d2vg,d2gg:d2gg,d2dd:d2dd}] &endfor 
uvstr=uvstr(1:*) & vgx=vgx(1:*)

save,bear,cpr,day,elon,entries,fbr,hdb,index,ktime,lsflg,minz,nlat,pdf,$
rmx,s,spdms,string,uvstr,vgx,zhour,zmin,elonk,nlatk,rmwk,filename='suv.idl'
END ;SUV.PRO
